/****************************************************************************
**  CUBE        http://www.scalasca.org/                                   **
*****************************************************************************
**  Copyright (c) 1998-2018                                                **
**  Forschungszentrum Juelich GmbH, Juelich Supercomputing Centre          **
**                                                                         **
**  This software may be modified and distributed under the terms of       **
**  a BSD-style license.  See the COPYING file in the package base         **
**  directory for details.                                                 **
****************************************************************************/


#include "config.h"


#include <cmath>
#include <QDebug>

#include "AggregatedTreeItem.h"
#include "CubeMetric.h"
#include "CubeProxy.h"
#include "CubeRegion.h"
#include "CubeIdIndexMap.h"

#include "MetricTree.h"
#include "Globals.h"
#include "TreeView.h"
#include "AggregatedTreeItem.h"

using namespace std;
using namespace cube;
using namespace cubegui;

MetricTree::MetricTree()
{
    treeType = METRICTREE;
}

QString
MetricTree::getLabel()
{
    if ( !cube )
    {
        return "not yet initialized";
    }
    // todo proxy return ( cube->getAttribute( "metrictree title" ) != "" ) ? cube->getAttribute( "metrictree title" ).c_str() : "Metric tree";
    return "Metric tree";
}

TreeItem*
MetricTree::createTree()
{
    TreeItem* top = new TreeItem( QString(), METRICITEM, 0 );

    vector<cube::Metric*> metrics = cube->getRootMetrics();

    if ( Globals::isExpertMode() ) // also show ghost metrics
    {
        vector<cube::Metric*> ghost = cube->getGhostMetrics();
        metrics.insert( metrics.end(), ghost.begin(), ghost.end() );
    }
    createItems<cube::Metric>( top, metrics, METRICITEM );

    return top;
}

/** return false, if the item is invalid and should not be added to the tree */
bool
MetricTree::itemIsValid( cube::Vertex* vertex )
{
    // if the metric item is VOID, it will not inserted into the tree
    cube::Metric* metric     = ( cube::Metric* )vertex;
    bool          voidValued = false;
    while ( true )
    {
        if ( std::strcmp( metric->get_val().c_str(), "VOID" ) == 0 )
        {
            voidValued = true;
            break;
        }
        metric = metric->get_parent();
        if ( metric == NULL )
        {
            break;
        }
    }
    return !voidValued;
}

QString
MetricTree::getItemName( cube::Vertex* vertex )
{
    return QString( ( ( cube::Metric* )vertex )->get_disp_name().c_str() );
}

double
MetricTree::getMaxValue( const TreeItem* item ) const
{
    assert( maxValueIndex.size() > 0 );
    return maxValues.at( maxValueIndex.value( item ) );
}

void
MetricTree::setBasicValues( const QList<Tree*>& leftTrees,
                            const QList<Tree*>& rightTrees )
{
    QList<TreeItem*> itemsToCalculate = getItemsToCalculate();
    if ( itemsToCalculate.size() == 0 )
    {
        return;
    }

    // Set up tree pointers
    Tree* callTree          = 0;
    Tree* callFlat          = 0;
    Tree* systemTree        = 0;
    bool  rightOfCallDim    = false;
    bool  rightOfSystemTree = false;


    // search in the left trees
    foreach( Tree * item, leftTrees )
    {
        if ( item->getTreeType() == CALLTREE )
        {
            callTree       = item;
            rightOfCallDim = true;
        }
        else if ( item->getTreeType() == CALLFLAT )
        {
            callFlat       = item;
            rightOfCallDim = true;
        }
        else if ( item->getTreeType() == SYSTEMTREE )
        {
            systemTree        = item;
            rightOfSystemTree = true;
        }
    }

    // search in the right trees
    if ( !systemTree || ( !callTree && !callFlat ) )
    {
        foreach( Tree * item, rightTrees )
        {
            if ( item->getTreeType() == CALLTREE )
            {
                callTree = item;
            }
            else if ( item->getTreeType() == CALLFLAT )
            {
                callFlat = item;
            }
            else if ( item->getTreeType() == SYSTEMTREE )
            {
                systemTree = item;
            }
        }
    }

    assert( callTree || callFlat );
    assert( systemTree );

    // Set up selection for system tree
    // if system tree is right, leave sysres_selection empty
    list_of_sysresources sysres_selection;
    if ( rightOfSystemTree )  // metric tree is on the right
    {
        QList< TreeItem* > selected_systemnodes = systemTree->getSelectionList();
        foreach( TreeItem * item, selected_systemnodes )
        {
            sysres_pair sp;
            sp.first  = static_cast<cube::Sysres*> ( item->getCubeObject() );
            sp.second = ( item->isExpanded() ) ? cube::CUBE_CALCULATE_EXCLUSIVE : cube::CUBE_CALCULATE_INCLUSIVE;
            sysres_selection.push_back( sp );
        }
    }
    else
    {
        sysres_selection = CubeProxy::ALL_SYSTEMNODES;
    }

    if ( callTree )
    {
        // Set up selection for call tree
        // if call tree is right, leave sysres_selection empty
        list_of_cnodes cnode_selection;

        if ( rightOfCallDim )  // metric tree is on the right
        {
            QList< TreeItem* > selected_cnodes = callTree->getSelectionList();
            foreach( TreeItem * item, selected_cnodes )
            {
                cnode_pair cp;
                cp.first  = static_cast<cube::Cnode*> ( item->getCubeObject() );
                cp.second = ( item->isExpanded() ) ? cube::CUBE_CALCULATE_EXCLUSIVE : cube::CUBE_CALCULATE_INCLUSIVE;
                cnode_selection.push_back( cp );
            }
        }
        else
        {
            cnode_selection = CubeProxy::ALL_CNODES;
        }

        std::vector< Value* > inclusive_values;
        std::vector< Value* > exclusive_values;
        QList<TreeItem*>      list = top->getChildren();

        int i     = 0;
        int step  = std::max( list.size() / PROGRESS_STEPS, 1 );
        int depth = 0; // 0 = first level; todo proxy optimize

        // request bulk data from CubeProxy
        while ( !list.isEmpty() )
        {
            if ( ++i % step == 0 )
            {
                QString progress = QString::number( i ) + "/" + QString::number( top->getChildren().size() );
                Globals::setStatusMessage( "Calculating " + this->getLabel() + " values (" + progress + ")", Information, false );
            }
            TreeItem* child = list.takeFirst();
            inclusive_values.clear();
            exclusive_values.clear();
            cube::IdIndexMap indexMap;
            cube::Metric*    metric = static_cast<cube::Metric*> ( child->getCubeObject() );
            cube->getMetricSubtreeValues( cnode_selection,
                                          sysres_selection,
                                          *metric,
                                          depth,
                                          indexMap,
                                          &inclusive_values,
                                          &exclusive_values );

            assignValues( child, inclusive_values, exclusive_values, child->getDepth() + depth, &indexMap );
            if ( child->isExpanded() )
            {
                foreach( TreeItem * it, child->getChildren() )
                list.prepend( it );
            }
        }
    }


    else if ( callFlat )
    {
        // Set up selection for call flat
        list_of_regions region_selection;

        // Marks that some regions got status "subroutines" and have t be reset after calculation
        bool unset_regions = false;

        if ( rightOfCallDim )  // metric tree is on the right
        {
            QList< TreeItem* > selected_regions = callFlat->getSelectionList();
            foreach( TreeItem * item, selected_regions )
            {
                region_pair rp;
                rp.first = static_cast<cube::Region*> ( item->getCubeObject() );
                if ( rp.first == NULL ) // subroutines
                {
                    unset_regions = true;
                    cube::Region* reg = static_cast<cube::Region*> ( item->getParent()->getCubeObject() );
                    reg->set_as_subroutines();
                    rp.first = reg;
                }
                rp.second = ( item->isExpanded() ) ? cube::CUBE_CALCULATE_EXCLUSIVE : cube::CUBE_CALCULATE_INCLUSIVE;
                region_selection.push_back( rp );
            }
        }
        else
        {
            // Flat tree selection is the inclusive root node(s)
            QList< TreeItem* > selected_regions = callFlat->getTop().getChildren();
            TreeItem*          item             = selected_regions.at( 0 );
            region_pair        rp;
            rp.first  = static_cast<cube::Region*> ( item->getCubeObject() );
            rp.second = cube::CUBE_CALCULATE_INCLUSIVE;
            region_selection.push_back( rp );
        }

        // request bulk data from CubeProxy
        vector< Value* > inclusive_values;
        vector< Value* > exclusive_values;

        // get values for full tree
        cube->getMetricTreeValues( region_selection,
                                   sysres_selection,
                                   inclusive_values,
                                   exclusive_values );


        foreach( TreeItem * child, top->getChildren() )
        {
            // Assign values to individual nodes
            assignValues( child, inclusive_values, exclusive_values );
        }

        if ( unset_regions )
        {
            for ( list_of_regions::iterator riter = region_selection.begin(); riter != region_selection.end(); ++riter )
            {
                ( *riter ).first->unset_as_subroutines();
            }
        }
    }
}



// -----------------------------------------------------------------
// ---------------- calculation functions --------------------------
// -----------------------------------------------------------------

void
MetricTree::computeMaxValues()
{
    double roundThreshold = Globals::getRoundThreshold( FORMAT_TREES );
    double value;
    double maxValue = 100.0;

    maxValues.clear();
    maxValueIndex.clear();

    // for metric trees we compute the max values for each root's subtree
    QList<TreeItem*> subTree;

    for ( unsigned i = 0; i < ( unsigned )top->getChildren().size(); i++ )
    {
        subTree.clear();
        subTree.append( top->getChildren().at( i ) );
        maxValue = top->getChildren().at( i )->totalValue;

        for ( int j = 0; j < subTree.size(); j++ )
        {
            maxValueIndex.insert( subTree[ j ], i );
            foreach( TreeItem * child, subTree[ j ]->getChildren() )
            {
                subTree.append( child );
            }
            getMax( subTree[ j ], value );
            // we save max absolute value (for the case of differences of cube)
            if ( fabs( maxValue ) < fabs( value ) )
            {
                maxValue = value;
            }
        }

        if ( fabs( maxValue ) <= fabs( roundThreshold ) )
        {
            maxValue = 0.0;
        }
        maxValues.append( maxValue );
    }
} // end of computeMaxValues()


TreeItem*
MetricTree::getTreeItem( std::string metricId )
{
    foreach( TreeItem * item, this->getItems() )
    {
        cube::Metric* metric = static_cast<cube::Metric*>( item->cubeObject );
        if ( metric->get_uniq_name() == metricId )
        {
            return item;
        }
    }
    return 0;
}





// return the name if the ith external metric item
//
double
MetricTree::getExternalReferenceValue( const QString& name )
{
    return externalMetricValue.value( name, 0.0 );
}


// compute the needed external values from the "cubeExternal"
// database for the external percentage value modus
//
void
MetricTree::computeExternalReferenceValues( cube::CubeProxy* cubeExternal )
{
    externalMetricValue.clear();

    const std::vector<cube::Metric*>& metricVec = cubeExternal->getMetrics();
    const std::vector<cube::Cnode*>&  callVec   = cubeExternal->getCnodes();
    const std::vector<cube::Thread*>& threadVec = cubeExternal->getLocations();

    // calculate inclusive values for all metric root items
    for ( int i = 0; i < ( int )metricVec.size(); i++ )
    {
        // take the next metric item
        cube::Metric* metric     = metricVec[ i ];
        QString       metricName = QString::fromStdString( metric->get_uniq_name() );

        // compute external metric values
        double totalValue = 0.0;
        for ( int j = 0; j < ( int )callVec.size(); j++ )
        {
            for ( int k = 0; k < ( int )threadVec.size(); k++ )
            {
                /// @fixme @todo Implement proxy computation for external reference values
                //totalValue += cube->get_sev( metric, callVec[ j ], threadVec[ k ] );
            }
        }
        externalMetricValue[ metricName ] = totalValue;
    }
}
