/****************************************************************************
**  CUBE        http://www.scalasca.org/                                   **
*****************************************************************************
**  Copyright (c) 1998-2018                                                **
**  Forschungszentrum Juelich GmbH, Juelich Supercomputing Centre          **
**                                                                         **
**  This software may be modified and distributed under the terms of       **
**  a BSD-style license.  See the COPYING file in the package base         **
**  directory for details.                                                 **
****************************************************************************/


#include "config.h"


#include "MainWidget.h"
#include "CubeError.h"
#include "PluginManager.h"
#include <iostream>
#include <iomanip>
#include <QFileDialog>
#include <QApplication>
#include "Globals.h"


using namespace std;
using namespace cubegui;

/** CubeApplication is required for OS X to open cube by double-clicking on a cube input file */
class CubeApplication : public QApplication
{
public:
    CubeApplication( int& argc, char** argv )
        : QApplication( argc, argv )
    {
    }

    void
    setMain( cubegui::MainWidget* m )
    {
        mainWidget = m;
    }

    bool
    event( QEvent* event )
    {
        if ( mainWidget && event->type() == QEvent::FileOpen )
        {
            QFileOpenEvent* openEvent = static_cast<QFileOpenEvent*>( event );
            mainWidget->loadFile( openEvent->file() );
        }
        return QApplication::event( event );
    }
private:
    cubegui::MainWidget* mainWidget;
};

// this is the main method
// handles command-line arguments and starts the GUI
int
main( int argc, char* argv[] )
{
    QAction*           startAction = 0;
    cube::CubeStrategy strategy    = cube::CUBE_ALL_IN_MEMORY_STRATEGY;

    CubeApplication app( argc, argv );

    cubegui::MainWidget* mainWidget     = NULL;
    bool                 disablePlugins = false;

    QStringList args = app.arguments();
    // set default name for settings
    QCoreApplication::setOrganizationName( "FZJ" );
    QCoreApplication::setApplicationName( "Cube" );

    args.removeAll( "" );
    if ( args.removeAll( "-disable-plugins" ) > 0 )
    {
        cubegui::PluginManager::disablePlugins();
        disablePlugins = true;
    }
    if ( args.removeAll( "-verbose" ) > 0 )
    {
        Globals::setVerbose( true );
    }
    if ( args.removeAll( "-expert" ) > 0 )
    {
        Globals::setExpertMode( true );
    }
    if ( args.removeAll( "-experimental" ) > 0 )
    {
        Globals::setExperimental( true );
    }
    if ( args.removeAll( "-preload" ) > 0 )
    {
        strategy = cube::CUBE_ALL_IN_MEMORY_PRELOAD_STRATEGY;
    }
    if ( args.removeAll( "-lastN" ) > 0 )
    {
        strategy = cube::CUBE_LAST_N_ROWS_STRATEGY;
    }
    if ( args.removeAll( "-start" ) > 0 ) // start a context free plugin
    {
        if ( args.size() < 2 )
        {
            std::cout << "cube -start <pluginname>: plugin name required" << endl;
            exit( 0 );
        }
        args.takeFirst();
        QString name = args.takeFirst();
        PluginManager::getInstance()->initializePlugins();
        QList<QAction*> actions = PluginManager::getInstance()->getContextFreeActions();
        foreach( QAction * action, actions )
        {
            if ( action->objectName() == name )
            {
                startAction = action;
                PluginManager::getInstance()->setPluginArguments( args );
                break;
            }
        }
    }
    QString systemtab;
    foreach( QString arg, args ) // -systemtab:tabname to start with selected systemtab
    {
        if ( arg.contains( "-systemtab:" ) )
        {
            args.removeAll( arg );
            systemtab = arg.replace( "-systemtab:", "" );
        }
    }
    if ( args.removeAll( "-help" ) > 0 || args.removeAll( "--help" ) > 0 || args.size() > 2 )
    {
        std::cout << "Usage: cube [-disable-plugins] [-verbose] [-preload] [-lastN] [-expert] [-help] filename\n";
        std::cout << "       cube                    [-verbose] [-preload] [-lastN] [-expert] [-help] -start <plugin name> [plugin args]\n";
        return 0;
    }

    try
    {
        mainWidget = new cubegui::MainWidget( strategy );
        app.setMain( mainWidget );

        mainWidget->initialSystemTab = systemtab;

        if ( !startAction && ( args.size() == 2 ) )
        {
            QString name = args[ 1 ];

            if ( name.startsWith( ( "cube://" ) ) )
            {
                mainWidget->loadFile( name );
            }
            else
            {
                // if the command line argument is a directory then
                // open a file dialog for getting the file name
                if ( !QFile( name ).exists() || QDir( name ).exists() )
                {
                    name = QFileDialog::getOpenFileName( NULL,
                                                         app.tr( "Choose a file to open" ),
                                                         name,
                                                         app.tr( "Cube3/4 files (*cube *cube.gz *.cubex);;Cube4 files (*.cubex);;Cube files (*.cube.gz *.cube);;All files (*.*);;All files (*)" ) );
                }
                if ( name.length() > 0 )
                {
                    mainWidget->loadFile( name );
                }
            }
            if ( disablePlugins )
            {
                Globals::setStatusMessage( MainWidget::tr( "plugins are disabled..." ) );
            }
        }
        mainWidget->show();
        if ( startAction )
        {
            startAction->activate( QAction::Trigger );
        }

        int return_code =  app.exec();

        delete mainWidget;
        return return_code;
    }
    catch ( const cube::RuntimeError& err )
    {
        delete mainWidget;
        std::cerr << err.what() << std::endl;
    }
    return -1;
}
