/****************************************************************************
**  CUBE        http://www.scalasca.org/                                   **
*****************************************************************************
**  Copyright (c) 1998-2019                                                **
**  Forschungszentrum Juelich GmbH, Juelich Supercomputing Centre          **
**                                                                         **
**  This software may be modified and distributed under the terms of       **
**  a BSD-style license.  See the COPYING file in the package base         **
**  directory for details.                                                 **
****************************************************************************/


#include "config.h"
#include <algorithm>
#include <vector>
#include <utility>
#include <cassert>

#include <QtPlugin>
#include <QDebug>
#include <QButtonGroup>
#include <QVBoxLayout>
#include <QFileDialog>
#include <QDirIterator>
#include <QFileInfo>
#include <QStringList>
#include <QApplication>
#include <QStyle>

#include "CubeScaling.h"
#include "ContextFreeServices.h"
#include "CubeProxy.h"
#include "Cube.h"
#include "algebra4.h"
#include "Globals.h"

using namespace cube;
using namespace cubepluginapi;
using namespace scaling;

#if QT_VERSION < 0x050000
Q_EXPORT_PLUGIN2( CubeScalingPlugin, CubeScaling ) // ( PluginName, ClassName )
#endif

void
CubeScaling::version( int& major, int& minor, int& bugfix ) const
{
    major  = 1;
    minor  = 0;
    bugfix = 0;
}

QString
CubeScaling::name() const
{
    return "Scaling";
}

void
CubeScaling::opened( ContextFreeServices* service )
{
    this->service = service;

    QWidget* widget = this->service->getWidget();

    QHBoxLayout* layoutOuter = new QHBoxLayout();
    widget->setLayout( layoutOuter );



    QWidget*     inner  = new QWidget();
    QVBoxLayout* layout = new QVBoxLayout( inner );
    layoutOuter->addWidget( inner );
    inner->setSizePolicy( QSizePolicy::Maximum, QSizePolicy::Maximum );

    cubes = new QListWidget();


    QPushButton* addCube   = new QPushButton( "Add cubes" );
    QPushButton* clearList = new QPushButton( "Clear list" );

    scaling = new QPushButton( "Scaling analysis" );
    connect( addCube,                   SIGNAL( clicked() ),    this, SLOT( selectCubes() ) );
    connect( clearList,                   SIGNAL( clicked() ),    this, SLOT( clearCubes() ) );
    connect( scaling,                     SIGNAL( clicked() ),    this, SLOT( startAction() ) );
    layout->addWidget( cubes );
    QButtonGroup* options = new QButtonGroup();
    aggregation = new QRadioButton( tr( "Aggregate over system (accounted resources)" ), widget );
    aggregation->setChecked( true );
    options->addButton( aggregation );
    maximum = new QRadioButton( tr( "Maximum over system tree (wallclock time)" ), widget );
    options->addButton( maximum );
    layout->addWidget( aggregation );
    layout->addWidget( maximum );


    layout->addWidget( addCube );
    layout->addWidget( clearList );
    layout->addWidget( scaling );

    scaling->setEnabled( false );
    scaling->setStyleSheet( "padding:4px;text-align: center" );
}


void
CubeScaling::startAction()
{
    std::vector<std::string> options;
    if ( aggregation->isChecked() )
    {
        options.push_back( "aggregate" );
    }
    std::vector<std::string> fileNames;
    for ( int i = 0; i < cubes->count(); i++ )
    {
        fileNames.push_back( cubes->item( i )->text().toStdString() );
    }
    try
    {
        cube = CubeProxy::create( ALGORITHM_SCALING, fileNames, options );
    }
    catch ( const RuntimeError& e )
    {
        std::cerr << e.what() << std::endl;
    }


    cube->defineAttribute( "origin", "scaling" ); // mark this cube with an attribure
    service->openCube( cube );
}


void
CubeScaling::selectCubes()
{
    QStringList experiments;
    QString     dir = QFileDialog::getExistingDirectory( service->getWidget(), tr( "Open Score-P Experiement " ),
                                                         "",
                                                         QFileDialog::ShowDirsOnly
                                                         | QFileDialog::DontResolveSymlinks );
    selectCubes( dir );
}


void
CubeScaling::selectCubes( const QString& dir )
{
    QStringList  experiments;
    QDirIterator it( dir, QDirIterator::Subdirectories );
    while ( it.hasNext() )
    {
        QString   file = it.next();
        QFileInfo fi( file );
        QString   name   = fi.baseName();
        QString   suffix = fi.completeSuffix();
        if ( ( suffix.compare( "cubex" ) == 0 ) && ( name.compare( "summary" ) == 0 ) )
        {
            experiments << file;
        }
    }
    cubes->addItems( experiments );
    scaling->setEnabled( cubes->count() >= 1 );
}



void
CubeScaling::clearCubes()
{
    cubes->clear();
    scaling->setEnabled( cubes->count() >= 1 );
}




void
CubeScaling::closed()
{
}

QString
CubeScaling::getHelpText() const
{
    return "This plugin gathers metrics data from series of scaling measurements and collects them in the resuling Scaling Tree. In couple with \"Jenga Fett\" it allows to study the scaling behaviour of the application.";
}
